package eduni.simanim;

import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Point;
import java.awt.Dimension;
import java.awt.Image;

/**
 * An animation entity, used to store display information about an entity
 * for the animation.
 * @version     1.0, 4 September
 * @author      Ross McNab
 */

public class Anim_entity {
  private String name;
  private int x, y;
  private int w, h;
  private Vector ports;
  private Vector params;
  private Image image = null;
  private String base_img_name;

  //
  // The EDL and EDF interface
  //

  // Constructor
  /** This constuctor should not be used directly, use the extended
   * constructor in eduni.simjava.Sim_entity instead.
   */
  public Anim_entity(String name, String base_img) {
    this.name = name;
    this.image = Sim_anim.get_image(base_img);
    this.base_img_name = base_img;
    ports = new Vector();
    params = new Vector();
    this.set_position(0,0);
  }

  /** This method should not be used directly. */
  public void add_port(Anim_port port) {
    ports.addElement(port);
    port.set_parent(this);
    port.update_position();
  }

  /** This method should not be used directly. */
  public void add_param(Anim_param param) {
    if(param.get_type() == Anim_param.STATE) {
      this.image = Sim_anim.get_image(base_img_name+"."+param.get_value());
      this.set_position(x,y);
    }
    params.addElement(param);
    param.set_parent(this);
    param.update_position();
  }

  /** This method should not be used directly. */
  public void set_position(int x, int y) {
    this.x = x;  this.y = y;
    if(image != null) {
      w = this.image.getWidth(null);
      h = this.image.getHeight(null);
    }
    // now tell all the ports we've moved
    for (Enumeration e = ports.elements() ; e.hasMoreElements() ;) {
       ((Anim_port)e.nextElement()).update_position();
    }
    // now tell all the params we've moved
    for (Enumeration e = params.elements() ; e.hasMoreElements() ;) {
       ((Anim_param)e.nextElement()).update_position();
    }
  }

  //
  // The package interface
  //

  // Called on each anim run to initialise the ents
  //void initialise() {
  //  Anim_port port;
  //  Anim_param param;
  //  Enumeration e;

  //  for (e = ports.elements() ; e.hasMoreElements() ;) {
  //     port = (Anim_port)e.nextElement();
  //     port.move_msg(-1.0);
  //  }
  //  for (e = params.elements() ; e.hasMoreElements() ;) {
  //     param = (Anim_param)e.nextElement();
  //     param.initialise();
  //     if(param.get_type() == Anim_param.STATE) {
  //       this.image = Sim_anim.get_image(base_img_name+"."+param.get_value());
  //       this.set_position(x,y);
  //     }
  //  }    
  //}

  // Update our internal parameters from the passed string
  void set_params(String param_string) {
    Anim_param param;
    StringTokenizer st;

    st = new StringTokenizer(param_string);
    for (Enumeration e = params.elements() ; e.hasMoreElements() ;) {
       param = (Anim_param)e.nextElement();
       if(!st.hasMoreTokens())
         System.out.println("Error: Anim_entity.set_params - not enought tokens");
       if(param.get_type() == Anim_param.STATE) {
         param.set_value(st.nextToken());
         this.image = Sim_anim.get_image(base_img_name+"."+param.get_value());
         this.set_position(x,y);
       } else {
         param.set_value(st.nextToken());
       }
    }    
  }

  void draw_messages(Graphics g) {
    // Call draw_messages(g) on all my ports
    for (Enumeration e = ports.elements() ; e.hasMoreElements() ;) {
       ((Anim_port)e.nextElement()).draw_messages(g);
    }
  }

  void draw(Graphics g) {
    // Draw me
    g.setColor(Color.lightGray);
    g.fillRect(x,y,w,h);
    g.drawImage(image,x,y,null);
    // Call draw on my ports
    for (Enumeration e = ports.elements() ; e.hasMoreElements() ;) {
       ((Anim_port)e.nextElement()).draw(g);
    }
    // Call draw on my parameters
    for (Enumeration e = params.elements() ; e.hasMoreElements() ;) {
       ((Anim_param)e.nextElement()).draw(g);
    }
  }

  // Some package access functions
  String get_name() { return(name); }
  Dimension get_size() { return new Dimension(w, h); }
  Point get_position() { return new Point(x, y); }

  // If we have a port called port_name then return it
  Anim_port find_port(String port_name) {
    Anim_port port;
    for (Enumeration e = ports.elements() ; e.hasMoreElements() ;) {
      port = (Anim_port)e.nextElement();
      if(port.get_name().equals(port_name)) return port;
    }
    return null;
  }
}
